@@ -38,3 +38,21 @@ class ZTF( Survey ):
                         mjd_range, skynoise_range, 
                         inplace=True, **kwargs)
         return this
+
+
+    def show(self, data=None, **kwargs):
+        """ shows the sky coverage 
+
+        **kwargs goes to ztffields.skyplot_fields
+        """
+        import ztffields
+        if data is None:
+            data = self.data
+        
+        datamain = data[data["fieldid"]<1000] # main grid
+        fieldid_s = datamain.groupby("expid").first().groupby("fieldid").size()
+
+        fig = ztffields.skyplot_fields(fieldid_s, 
+                                        label="number of observations (main grid)",
+                                           **kwargs)
+
